function plotFctsSlepianCap( L, Theta, num_fcts, spectrum_end, save_files)
%
% plotFctsSlepianCap( L, Theta, num_fcts, spectrum_end)
%
% Plots the \num_fcts first or last spherical Slepian functions and save
% them if requested.
%
% @param L              Bandlimit in the Spherical Harmonics domain
% @param Theta          Maximum latitude of the spherical cap
% @param num_fcts       Number of basis functions to plot
% @param spectrum_end   From which end of the spectrum the basis functions
%                       are to be plotted
% @param save_files     Save image files for basis functions (eps + pdf)
%
% Original header (Our functions is adapted from simons sdwfried function):
% 
% SDWFRIED
%
% Makes fried-egg plots for +- angular orders
% Figure 5.4 from  Simons et al., SIAM Review (2006)
%
% Run twice. Second time round the color axis is OK
%
% Last modified by fjsimons-at-alum.mit.edu, 04/13/2009

  defval( 'L', 20);
  defval( 'Theta', 15);
  defval( 'num_fcts', 10);
  defval( 'spectrum_end', 'first');
  defval( 'save_files', 0);
  
  [lrnk,mrnk,lval]=sdwelm(Theta,L);

  % Nearly double the amount of requested tapers
  lrnk=gamini(lrnk,(mrnk~=0)+1);
  lval=gamini(lval,(mrnk~=0)+1);
  mrnk=gamini(mrnk,(mrnk~=0)+1);
  % This trick only works for the single cap
  mrnk((diff(mrnk)==0))=-mrnk((diff(mrnk)==0));

  % The indexing is not entirely clear so start from the data returned by
  % sdwelm()
  if( strcmp( spectrum_end, 'first'))
    basis_fct_ind = [1:num_fcts];  
  else    
    basis_fct_ind = [length(lval)-num_fcts+1:length(lval)];  
  end
  
  for index = basis_fct_ind
    
    figure;
    fnpl=sprintf('%s/SDWFRIED-%i-%i-%i.mat',...
           fullfile(getenv('IFILES'),'./data/'),Theta,L,index);
    if exist(fnpl,'file')==2
      load(fnpl)
      disp(sprintf('%s loaded by SDWFRIED',fnpl))
      cbb=gray(10);
      cbb=kelicol;
      ispl=data/max(abs(data(:)));
      ispl(abs(ispl)<1/100)=NaN;
      imagefnan([-1 1],[1 -1],ispl,cbb,[-1 1])
      colormap(cbb); hold on
      ax2(1)=circ(1); ax2(2)=circ(sin(Theta*pi/180));
      set(ax2(1:2),'LineW',1)
      set(ax2(2),'LineS','--')
      axis([-1.0100    1.0100   -1.0100    1.0100])
      axis off
      if index==0
        [axlim,handl,XYZ]=plotcont([0 90],[360 -90],4);
        delete(handl)
        hold on
        p1=plot(XYZ(:,1),XYZ(:,2),'k-','LineWidth',1);
      end
      title(sprintf('%s = %g', '\lambda', lval(index)));
      drawnow
      
    else
      
      % generate plot of basis function and save
      [E,V,N,th,C]=sdwcap(Theta,L,mrnk(index),128,[],2);
      E=E{lrnk(index)};
      Emax=max(E(:)); thresh=100;
      Ed=E;
      Ed(abs(Ed)<Emax/thresh)=0;
      [data,ch,ph]=plotplm(Ed,[],[],5,[],Theta);
      title( sprintf( '%s = %g', '\lambda', lval(index)));
      delete(ch)
      drawnow;
      save(fnpl,'data');
      close;
      
      % display the file loaded from disk to avoid a problem with the color
      % scaling (as also mentioned by Simons in the original header)
      figure;
      fnpl=sprintf('%s/SDWFRIED-%i-%i-%i.mat',...
           fullfile(getenv('IFILES'),'./data/'),Theta,L,index);
      load(fnpl)
      disp(sprintf('%s loaded by SDWFRIED',fnpl))
      cbb=gray(10);
      cbb=kelicol;
      ispl=data/max(abs(data(:)));
      ispl(abs(ispl)<1/100)=NaN;
      imagefnan([-1 1],[1 -1],ispl,cbb,[-1 1])
      colormap(cbb); hold on
      ax2(1)=circ(1); ax2(2)=circ(sin(Theta*pi/180));
      set(ax2(1:2),'LineW',1)
      set(ax2(2),'LineS','--')
      axis([-1.0100    1.0100   -1.0100    1.0100])
      axis off
      if index==0
        [axlim,handl,XYZ]=plotcont([0 90],[360 -90],4);
        delete(handl)
        hold on
        p1=plot(XYZ(:,1),XYZ(:,2),'k-','LineWidth',1);
      end
      title(sprintf('%s = %g', '\lambda', lval(index)));
      drawnow
      
    end

    % save image files if requested
    if( 1 == save_files)
      Save files in EPS and PDF format
      save file as an eps
      fname = ['./', 'fig_L', num2str(L), '_T', num2str(Theta), '_'];
      if( index < 10)
        fname = [fname '0'];    
      end
      fname = [fname, num2str(index)];
      print('-depsc', fname);
      print('-dpng', fname);
      create PDF file
      [s, w] = unix( ['epstopdf ' fname '.eps']);
    end
    
  end

end

