function omega = uniformPointsS2( total_samples, method, param )
%
% omega = uniformPointsS2( total_samples, method, theta)
%
% Create approximately uniformly distributed points on the sphere
% 
% @return  omega  total_samples uniformly distributed points on the sphere
%                 in standard spherical coordinates (theta,phi)
%
% @param num_samples  Number of random points to generate
% @param method   Random number generator to use for generating random 
%                 points on the sphere ('halton', 'spiral', 'sobol', 
%                 'sobol2', 'nie', 'random', 'equator', 'semi-equator';
%                 default is 'halton')
% @param param    'equi-latitude':latitude for generating equi-latitude 
%                 points 
%                 'random': seed for random number generator

  defval( 'method', 'halton');
  
  switch method
    case 'halton'
      omega=halton(total_samples,2);
      omega(:,1) = pi/2 - asin(omega(:,1));
      omega(:,2) = 2 * pi * omega(:,2);
    case 'halton-sphere'
      omega=halton(total_samples,2);
      omega(:,1) = asin((2*omega(:,1))-1.0)+pi/2;
      omega(:,2) = 2 * pi * omega(:,2);
    case 'spiral'
      omega = spiralPoints( total_samples+2);
      % skip the poles which can lead to points where Spherical Harmonics
      % cannot be evaluated accurately when used with optimization
      omega = omega( 2:end-1, :);
    case 'spiral-with-pole'
      omega = spiralPoints( total_samples+1);
      % skip only the last point which is always the north pole and
      % combined with the first point of the sequence which is the south
      % pole would lead to two identical kernels
      omega = omega( 1:end-1, :);
    case 'sobol'
      % choose points using Sobol quasi random sequence
      % this was exported to a file in case MATLAB implementation
      % doesn't have this MEX extension
      % load sobol_dat
      % omega = sobol_dat(:,1:total_samples)';
      omega = sobol( 2, total_samples, 'init')';
      omega(:,1) = pi/2 - asin(omega(:,1));
      omega(:,2) = 2 * pi * omega(:,2);
    case 'nie'
      % choose points using Niederreiter quasi random sequence
      % load nie_dat
      % omega = nie_dat(:,1:total_samples)';
      omega = niederreiter( 2, total_samples, 'init')';
      omega(:,1) = pi/2 - asin(omega(:,1));
      omega(:,2) = 2 * pi * omega(:,2);
    case 'equator'
      % choose points on the equator
      delta = 2*pi / total_samples;
      omega = zeros( total_samples, 2);
      omega(:,1) = pi/2;
      epsilon = 10^-6;
      omega(:,2) = [0 : delta : (2*pi-epsilon)];
    case 'semi-equator'
      defval( 'param', pi/3);
      % choose points on the equator
      omega = zeros( total_samples, 2);
      omega(:,1) = param;
      epsilon = 10^-6;
      delta = 2*pi / total_samples;
      omega(:,2) = [0 : delta : (2*pi-epsilon)];      
    case 'random'
      % 5489 sets random number generator to default value
      rand('twister', 5489);
      omega = rand( total_samples, 2);
      omega(:,1) = pi/2 - asin(omega(:,1));
      omega(:,2) = 2 * pi * omega(:,2);
    case 'cluster1'
      % points in subset of top hemisphere
      omega=halton(total_samples,2);      
      omega(:,1) = (8/10) * (pi/2 - asin(omega(:,1)));
      omega(:,2) = omega(:,2) * 2 * pi;
    case 'cluster2'
      % points in subset of top hemisphere
      omega=halton(total_samples,2);      
      omega(:,1) = (7/10) * (pi/2 - asin(omega(:,1)));
      omega(:,2) = omega(:,2) * 2 * pi;
    case 'cluster3'
      % points in subset of top hemisphere
      omega=halton(total_samples,2);      
      omega(:,1) = (2/5) * (pi/2 - asin(omega(:,1)));
      omega(:,2) = omega(:,2) * 2 * pi;
    case 'cap'
      % points in a spherical cap (param is expected to be less than pi/2)
      defval( 'param', deg_to_rad(10));
      omega = halton(total_samples,2);
      % rescale omega(:,1) so that it matches the desired range
      omega(:,1) = 1.0 - omega(:,1) * (1 - sin(pi/2 - param));
      omega(:,1) = pi/2 - asin(omega(:,1));
      omega(:,2) = omega(:,2) * 2 * pi;
    otherwise
      error('Unknown method "%s".', method);
  end
  
end